import { h, ref } from "vue";

export function getStyle(className: string): string {
  let cssText = "";
  for (let si = 0; si < document.styleSheets.length; si++) {
    let classes: any =
      document.styleSheets[si].rules || document.styleSheets[0].cssRules;
    for (let x = 0; x < classes.length; x++) {
      if (classes[x].selectorText == className) {
        cssText += classes[x].cssText || classes[x].style.cssText;
      }
    }
  }
  return cssText;
}

export class DomImage {
  img: any;

  constructor(width: number, height: number) {
    this.img = document.createElement("img");
    this.img.setAttribute("width", width);
    this.img.setAttribute("height", height);
  }

  set src(url: any) {
    this.img.src = url;
  }

  get naturalWidth(): number {
    return this.img.naturalWidth();
  }

  get naturalHeight(): number {
    return this.img.naturalHeight();
  }

  fromStyle(selector: string) {
    return new Promise((resolve: any) => {
      const style: any = getStyle(selector);
      const imgurlMatch = style.match(/url\(['"](.*?)['"]/);
      if (!imgurlMatch) {
        console.error("url for css selector", selector, "not found in", style);
        return;
      }
      this.img.onload = () => resolve(this.img);
      this.img.src = imgurlMatch[1];
    });
  }
}

export function getelse(obj: any, prop: any, def: any): any {
  if (typeof obj[prop] !== "undefined") {
    return obj[prop];
  }
  return def;
}

export class Canvas {
  width: number;
  height: number;
  ref: any;

  arrow(from: any, to: any, argsopt: any) {
    let diff: any = { x: to.x - from.x, y: to.y - from.y };
    let l = Math.sqrt(diff.x * diff.x + diff.y * diff.y);
    let rot = Math.asin((to.y - from.y) / l);
    if (to.x < from.x) rot = Math.PI - rot;
    let args = argsopt || {};
    let scalefactor = getelse(args, "scalefactor", 1);
    let auxscalefactor = getelse(args, "auxscalefactor", 1);
    let linewidth =
      getelse(args, "linewidth", 16) * scalefactor * auxscalefactor;
    let halflinewidth = linewidth / 2;
    let pointheight =
      getelse(args, "pointheight", 40) * scalefactor * auxscalefactor;
    let pointwidth =
      getelse(args, "pointwidth", 30) * scalefactor * auxscalefactor;
    let halfpointwidth = pointwidth / 2;
    let color = getelse(args, "color", "#ff0");
    let opacity = getelse(args, "opacity", 1);
    let lineheight = l - pointheight;
    this.ctx.save();
    this.ctx.globalAlpha = opacity;
    this.ctx.translate(from.x, from.y);
    this.ctx.rotate(rot);
    this.ctx.fillStyle = color;
    this.ctx.beginPath();
    this.ctx.moveTo(0, 0);
    this.ctx.lineTo(0, halflinewidth);
    this.ctx.lineTo(lineheight, halflinewidth);
    this.ctx.lineTo(lineheight, halflinewidth + halfpointwidth);
    this.ctx.lineTo(l, 0);
    this.ctx.lineTo(lineheight, -(halflinewidth + halfpointwidth));
    this.ctx.lineTo(lineheight, -halflinewidth);
    this.ctx.lineTo(0, -halflinewidth);
    this.ctx.lineTo(0, 0);
    this.ctx.closePath();
    this.ctx.fill();
    this.ctx.restore();
  }

  uploadHref(kind: string) {
    const href = this.ref._rawValue.toDataURL("image/" + kind);
    const b64 = href.replace(/data:image\/png;base64,/, "");
    return b64;
  }

  downloadHref(name: string, kind: string) {
    let dt: any = this.ref._rawValue.toDataURL("image/" + kind);
    dt = dt.replace(/^data:image\/[^;]*/, "data:application/octet-stream");
    dt = dt.replace(
      /^data:application\/octet-stream/,
      "data:application/octet-stream;headers=Content-Disposition%3A%20attachment%3B%20filename=" +
        name +
        "." +
        kind
    );
    return dt;
  }

  drawImage(img: any, x: number, y: number, width: number, height: number) {
    this.ctx.drawImage(img, x, y, width, height);
  }

  constructor(width: number, height: number) {
    this.width = width;
    this.height = height;
    this.ref = ref(0);
  }

  get ctx(): any {
    return this.ref._rawValue.getContext("2d");
  }

  draw() {
    this.ctx.moveTo(0, 0);
    this.ctx.lineTo(200, 100);
    this.ctx.stroke();
  }

  render(): any {
    return h(
      "canvas",
      {
        width: this.width,
        height: this.height,
        ref: this.ref,
      },
      []
    );
  }

  resize(width: number, height: number): Canvas {
    this.width = width;
    this.height = height;
    this.ref._rawValue.setAttribute("width", this.width);
    this.ref._rawValue.setAttribute("height", this.height);
    return this;
  }
}
